unit UAxis_Iso;
{$I TeeDefs.inc}

interface

uses
{$IFNDEF LINUX}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes,
{$IFDEF CLX}
  QGraphics, QControls, QForms, QDialogs, QExtCtrls, QStdCtrls,
{$ELSE}
  Graphics, Controls, Forms, Dialogs, ExtCtrls, StdCtrls,
{$ENDIF}
  System.Types, VclTee.TeeProcs, VclTee.TeEngine, VclTee.Chart, VclTee.Series, VclTee.TeCanvas, System.Math;

var
  isHorizPositionEnable, isVertPositionEnable: boolean;

procedure MakeIsoAxis_2D(HorizAxis, VertAxis: TChartAxis; HorizPos, VertPos: integer); overload;
procedure MakeIsoAxis_2D(aSeries: TChartSeries; HorizPos, VertPos: integer); overload;
procedure MakeIsoAxis_2D(var aChart: TChart; HorizPos, VertPos: integer); overload;
procedure UnMakeIsoAxis_2D(HorizAxis, VertAxis: TChartAxis); overload;
procedure UnMakeIsoAxis_2D(aSeries: TChartSeries); overload;
procedure UnMakeIsoAxis_2D(var aChart: TChart); overload;
procedure TheChart_IsoAxis(var TheChart: TChart; isIsoAxis: boolean; TheHorizPos, TheVertPos: integer);
procedure SubChart_SizePostion(var TheChart, SubChart: TChart);

implementation

var
  HorizRange, VertRange, Offset, HorizAxisScale, VertAxisScale, XYScreen: Double;

function GetXYScreen(aChart: TCustomAxisPanel): Double;
begin
  with aChart do
    begin
{$IFDEF CLX}
      Result := 1024.0 / 768.0;
{$ELSE}
      Result := 1.0 * (GetDeviceCaps(Canvas.Handle, HORZSIZE) / Screen.Width) / (GetDeviceCaps(Canvas.Handle, VERTSIZE) / Screen.Height);
{$ENDIF}
    end;
end;

procedure MakeIsoAxis_2D(HorizAxis, VertAxis: TChartAxis; HorizPos, VertPos: integer); overload;
var
  aMin, aMax: Double;
begin
  with VertAxis.ParentChart do
    begin
      if (ChartHeight > 0) and (ChartWidth > 0) then
        begin
          with HorizAxis do
            begin
              Automatic := true;
              CalcMinMax(aMin, aMax);
              SetMinMax(aMin, aMax);
              HorizRange := Maximum - Minimum;
            end;
          with VertAxis do
            begin
              Automatic := true;
              CalcMinMax(aMin, aMax);
              SetMinMax(aMin, aMax);
              VertRange := Maximum - Minimum;
            end;

          XYScreen := GetXYScreen(VertAxis.ParentChart);
          HorizAxisScale := (HorizRange / ChartWidth);
          VertAxisScale := (VertRange / ChartHeight) * XYScreen;
          if VertAxisScale > HorizAxisScale then
            begin
              isHorizPositionEnable := true;
              isVertPositionEnable := false;
              if HorizAxisScale <> 0 then
                begin
                  Offset := ((HorizRange * VertAxisScale / HorizAxisScale) - HorizRange) / 2.0;
                  with HorizAxis do
                    begin
                      if HorizPos = 0 then { Series Left }
                        SetMinMax(Minimum, Maximum + 2.0 * Offset)
                      else if HorizPos = 1 then { Series Center }
                        SetMinMax(Minimum - Offset, Maximum + Offset)
                      else if HorizPos = 2 then { Series Right }
                        SetMinMax(Minimum - 2.0 * Offset, Maximum);
                    end;
                end;
            end
          else if HorizAxisScale > VertAxisScale then
            begin
              isHorizPositionEnable := false;
              isVertPositionEnable := true;
              if VertAxisScale <> 0 then
                begin
                  Offset := ((VertRange * HorizAxisScale / VertAxisScale) - VertRange) / 2.0;
                  with VertAxis do
                    begin
                      if VertPos = 0 then { Series Top }
                        SetMinMax(Minimum, Maximum + 2.0 * Offset)
                      else if VertPos = 1 then { Series Center }
                        SetMinMax(Minimum - Offset, Maximum + Offset)
                      else if VertPos = 2 then { Series Bottom }
                        SetMinMax(Minimum - 2.0 * Offset, Maximum);
                    end;
                end;
            end;
        end;
    end;
end;

procedure MakeIsoAxis_2D(aSeries: TChartSeries; HorizPos, VertPos: integer); overload;
begin
  MakeIsoAxis_2D(aSeries.GetHorizAxis, aSeries.GetVertAxis, HorizPos, VertPos);
end;

procedure MakeIsoAxis_2D(var aChart: TChart; HorizPos, VertPos: integer); overload;
begin
  if aChart.SeriesCount > 0 then
    MakeIsoAxis_2D(aChart.Series[0], HorizPos, VertPos);
end;

procedure UnMakeIsoAxis_2D(HorizAxis, VertAxis: TChartAxis); overload;
var
  aMin, aMax: Double;
begin
  with VertAxis.ParentChart do
    begin
      if (ChartHeight > 0) and (ChartWidth > 0) then
        begin
          isHorizPositionEnable := false;
          isVertPositionEnable := false;
          with HorizAxis do
            begin
              Automatic := true;
              CalcMinMax(aMin, aMax);
              SetMinMax(aMin, aMax);
            end;
          with VertAxis do
            begin
              Automatic := true;
              CalcMinMax(aMin, aMax);
              SetMinMax(aMin, aMax);
            end;
        end;
    end;
end;

procedure UnMakeIsoAxis_2D(aSeries: TChartSeries); overload;
begin
  UnMakeIsoAxis_2D(aSeries.GetHorizAxis, aSeries.GetVertAxis);
end;

procedure UnMakeIsoAxis_2D(var aChart: TChart); overload;
begin
  if aChart.SeriesCount > 0 then
    UnMakeIsoAxis_2D(aChart.Series[0]);
end;

procedure TheChart_IsoAxis(var TheChart: TChart; isIsoAxis: boolean; TheHorizPos, TheVertPos: integer);
begin
  if isIsoAxis then
    MakeIsoAxis_2D(TheChart, TheHorizPos, TheVertPos)
  else
    UnMakeIsoAxis_2D(TheChart);
end;

procedure SubChart_SizePostion(var TheChart, SubChart: TChart);
const
  SubChartToEdge = 0;
  MinSizeRatio = 0.3;
var
  tmpMin, tmpMax, SubHorizRange, SubVertRange, SubOffset: Double;
  SubLeft, SubTop, SubRight, SubBottom, SubChartWidth, SubChartHeight: integer;
begin
  if (TheChart = nil) or (SubChart = nil) then
    exit;

  with TheChart do
    begin
      if (ChartWidth > 0) and (ChartHeight > 0) and (SeriesCount > 0) then
        begin
          { TheChart's AxisScale }
          with Series[0] do
            begin
              with GetHorizAxis do
                HorizRange := Maximum - Minimum;
              with GetVertAxis do
                VertRange := Maximum - Minimum;
              XYScreen := GetXYScreen(TheChart);
              HorizAxisScale := (HorizRange / ChartWidth);
              VertAxisScale := (VertRange / ChartHeight) * XYScreen;
            end;

          { According to TheChart's AxisScale, calculate SubChart's Width and Height. }
          with SubChart.Series[0].GetHorizAxis do
            begin
              Automatic := true;
              CalcMinMax(tmpMin, tmpMax);
              SetMinMax(tmpMin, tmpMax);
              SubHorizRange := Maximum - Minimum;

              SubChartWidth := Round(SubHorizRange / HorizAxisScale);
              if SubChartWidth / ChartWidth < MinSizeRatio then
                SubChartWidth := Round(ChartWidth * MinSizeRatio);
              SubOffset := (HorizAxisScale * SubChartWidth - SubHorizRange) / 2.0;
              SetMinMax(tmpMin - SubOffset, tmpMax + SubOffset);
              Increment := Series[0].GetHorizAxis.Increment;
            end;
          with SubChart.Series[0].GetVertAxis do
            begin
              Automatic := true;
              CalcMinMax(tmpMin, tmpMax);
              SetMinMax(tmpMin, tmpMax);
              SubVertRange := Maximum - Minimum;

              SubChartHeight := Round(SubVertRange / VertAxisScale);
              if SubChartHeight / ChartHeight < MinSizeRatio then
                SubChartHeight := Round(ChartHeight * MinSizeRatio);
              SubOffset := (VertAxisScale * SubChartHeight / XYScreen - SubVertRange) / 2.0;
              SetMinMax(tmpMin - SubOffset, tmpMax + SubOffset);
              Increment := Series[0].GetVertAxis.Increment;
            end;

          { Put SubChart in the top right corner }
          SubRight := ChartRect.Right - SubChartToEdge;
          SubTop := ChartRect.Top + SubChartToEdge;
          SubLeft := SubRight - SubChartWidth;
          SubBottom := SubTop + SubChartHeight;
          with SubChart do
            begin
              CustomChartRect := true;
              ChartRect := TeeRect(SubLeft, SubTop, SubRight, SubBottom);
            end;
        end;
    end;
end;

end.
