unit UPlot_Func;

interface

uses
  Winapi.Windows, Winapi.Messages, System.Variants, System.Classes, System.Types, System.SysUtils, System.StrUtils, Data.DB, Data.Win.ADODB,
  Vcl.ComCtrls, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.Controls, Vcl.Menus, Vcl.Forms, Vcl.Dialogs, Vcl.Buttons, Vcl.ExtDlgs, Vcl.Graphics, Vcl.Imaging.jpeg,
  VclTee.TeEngine, VclTee.Series, VclTee.TeeComma, VclTee.TeeGDIPlus, VclTee.TeeProcs, VclTee.Chart, VclTee.TeePolar, VclTee.TeeLisB, VclTee.TeeTools,
  VclTee.TeeArrowTool, VclTee.TeeEdit, VclTee.TeCanvas, VclTee.TeePenDlg, VclTee.TeeEdiGrad, VclTee.EditChar, VclTee.TeeCustomLegendTool,
  VclTee.TeePoin3, VclTee.TeeSurfa, VclTee.TeeEdiGene, VclTee.TeePrevi, VclTee.TeePreviewPanel, System.Math, UAxis_Iso;

const
  L_Zero = 1E-2;
  A_Zero = 1E-4;
  ChartMargin = 10;
  AxisMinMaxOffset = 10;
  TitleFontSize = 12;
  AxisTitleFontSize = 9;
  AxisLabelFontSize = 8;
  FrameWidth = 2;
  AxisWidth = 2;
  LineWidth = 2;

var
  tmpXVal, tmpYVal: Double;
  TheChartSeriesGroup, SubChartSeriesGroup: TSeriesGroup;

procedure Setup_Chart_Axes(var aChart: TChart);
procedure TheChart_Axes(var aChart: TChart);
procedure SubChart_Axes(var aChart: TChart);
procedure Add_LineSeries(var aChart: TChart; aGroup: TSeriesGroup; isTheChart: boolean; aSeriesTitle: string; aSeriesColor: TColor;
  isInLegend: boolean);
procedure LineSeries_Data(var aChart: TChart; isTheChart: boolean; SeriesNo: integer);

implementation

procedure Setup_Chart_Axes(var aChart: TChart);
var
  i: integer;
begin
  with aChart do
    begin
      View3D := false;
      Walls.Visible := false;
      BevelInner := bvNone;
      BevelOuter := bvNone;
      Frame.Color := clBlack; { The frame is displayed using the Chart.ChartRect coordinates }
      Frame.Width := FrameWidth;
      Gradient.Visible := false;
      Shadow.Visible := false;
      MarginUnits := muPixels; { muPercent, muPixels }
      MarginTop := ChartMargin;
      MarginBottom := ChartMargin;
      MarginLeft := ChartMargin;
      MarginRight := ChartMargin;
      Title.Visible := false;
      Foot.Visible := true;
      Foot.Font.DefaultFont := false;
      Foot.Font.Color := clBlack;
      Foot.Font.Size := TitleFontSize;
      Foot.Font.Style := [fsBold];
      Legend.Shadow.Visible := false;
      Legend.CustomPosition := true;
      Legend.LegendStyle := lsSeries;
      Legend.FontSeriesColor := true;
      Legend.Font.DefaultFont := false;
      Legend.Font.Size := AxisLabelFontSize;
      Legend.Font.Style := [];
      for i := 0 to Axes.Count - 1 do
        begin
          with Axes[i] do
            begin
              PositionUnits := muPixels;
              Axis.Width := AxisWidth;
              Axis.Color := clBlack;
              MinimumRound := true;
              MaximumRound := true;
              RoundFirstLabel := true;
              Title.Visible := true;
              Title.Font.DefaultFont := false;
              Title.Font.Size := AxisTitleFontSize;
              Title.Font.Style := [];
              Labels := true;
              LabelsFont.DefaultFont := false;
              LabelsFont.Size := AxisLabelFontSize;
              LabelsFont.Style := [];
              MinorTickCount := 3;
              AxisValuesFormat := '#,##0.##';
            end;
        end;
      CustomAxes.Clear;
      if SeriesCount > 0 then
        FreeAllSeries();
    end;
end;

procedure TheChart_Axes(var aChart: TChart);
begin
  with aChart do
    begin
      with TopAxis do
        begin
          Title.Caption := 'Horizontal Axis (m)';
          Inverted := false;
        end;
      with LeftAxis do
        begin
          Title.Caption := 'Vertical Axis (m)';
          Inverted := true;
        end;
      TopAxis.UsePartnerAxis := true; { Use of elected Partner Axes to limit GridLine zone }
      TopAxis.PartnerAxis := LeftAxis;
      LeftAxis.UsePartnerAxis := true;
      LeftAxis.PartnerAxis := TopAxis;
    end;
end;

procedure SubChart_Axes(var aChart: TChart);
begin
  with aChart do
    begin
      with BottomAxis do
        begin
          Title.Caption := 'Abscissa (m)';
          Inverted := false;
        end;
      with LeftAxis do
        begin
          Title.Caption := 'Ordinate (m)';
          Inverted := false;
        end;
      BottomAxis.UsePartnerAxis := true; { Use of elected Partner Axes to limit GridLine zone }
      BottomAxis.PartnerAxis := LeftAxis;
      LeftAxis.UsePartnerAxis := true;
      LeftAxis.PartnerAxis := BottomAxis;
    end;
end;

procedure Add_LineSeries(var aChart: TChart; aGroup: TSeriesGroup; isTheChart: boolean; aSeriesTitle: string; aSeriesColor: TColor;
  isInLegend: boolean);
begin
  with aChart.AddSeries(TLineSeries) as TLineSeries do
    begin
      Clear;
      Title := aSeriesTitle;
      if isTheChart then
        HorizAxis := aTopAxis
      else
        HorizAxis := aBottomAxis;
      VertAxis := aLeftAxis;
      TreatNulls := tnDontPaint; { tnDontPaint/tnSkip/tnIgnore }
      // DrawStyle := aDrawStyle;
      XValues.Order := loNone;
      YValues.Order := loNone;
      LinePen.Visible := true;
      LinePen.Style := psSolid;
      LinePen.Width := LineWidth;
      SeriesColor := aSeriesColor;
      LinePen.Color := SeriesColor;
      Pointer.Visible := true;
      Pointer.Style := psNothing;
      Pointer.Color := SeriesColor;
      ShowInLegend := isInLegend;
      aGroup.Add(aChart[aChart.SeriesCount - 1]);
    end;
end;

procedure LineSeries_Data(var aChart: TChart; isTheChart: boolean; SeriesNo: integer);
var
  k: integer;
begin
  with aChart do
    begin
      k := SeriesCount;
      if isTheChart then
        begin
          Series[k - 1].AddXY(0, 0, '', clTeeColor);
          Series[k - 1].AddXY(250 * (SeriesNo + 1), 300 * (SeriesNo + 1), '', clTeeColor);
          Series[k - 1].AddXY(500 * (SeriesNo + 1), 500 * (SeriesNo + 1), '', clTeeColor);
        end
      else
        begin
          Series[k - 1].AddXY(0, 0, '', clTeeColor);
          Series[k - 1].AddXY(150 * (SeriesNo + 1), 200 * (SeriesNo + 1), '', clTeeColor);
          Series[k - 1].AddXY(250 * (SeriesNo + 1), 250 * (SeriesNo + 1), '', clTeeColor);
        end;
    end;
end;

end.
